
* example data set with IDs and addresses;
data addresses;
input 
@01 id      $5.
@10 addr1  $30.
@40 addr2  $30.
;
datalines;
12345    59 LENOX AVE 12203            1 UNIVERSITY PL 12144
23456    61 ACADEMY RD 12208           59 LENOX AVE 12203
99999    45 ROCKFELLER PLAZA 10111     116th ST 10027
;
run;
 
* place number of addresses in a macro variable;
data _null_;
call symputx('naddr',obs);
stop;
set addresses nobs=obs;
run;
 
 
* use a loop within a macro to access Google Maps multiple time;
%macro distance_time(type);
* delete any data set named DISTANCE_TIME that might exist in the WORK library;
proc datasets lib=work nolist;
delete distance_time;
quit;

%do j=1 %to &naddr;
data _null_;
nrec = &j;
set addresses point=nrec;
call symputx('id',id);
call symputx('a1',translate(trim(addr1),'+',' '));
call symputx('a2',translate(trim(addr2),'+',' '));
stop;
run;
 
filename x url "http://maps.google.com/maps?daddr=&a1=%nrstr(&saddr)=&a2%nrstr(&dirflg)=&type";
filename z temp;
 
data _null_; 
infile x recfm=f lrecl=1 end=eof; 
file z recfm=f lrecl=1;
input @1 x $char1.; 
put @1 x $char1.;
if eof;
call symputx('filesize',_n_);
run;
 
data temp;
length addr1 addr2 $30;
id = "&id";
addr1 = upcase(translate("&a1",' ','+'));
addr2 = upcase(translate("&a2",' ','+'));
 
infile z recfm=f lrecl=&filesize. eof=done;

%if &type ^= tr %then %do;
input @ "distance:'" @;
input text $50.;
distance = input(scan(text,1,"' "),comma12.);
units    = scan(text,2,"' ");
text     = scan(text,3,"'"); 
%end;
%else %do;
input @ "Travel time: about" @;
input text $50.;
text = scan(text,2,'<>'); 
%end;

select;
 when (find(text,'day') ne 0) 
  time = 86400*input(scan(text,1,' '),best.)+3600*input(scan(text,3,' '),best.);
 when (find(text,'hour') ne 0)  
  time = 3600*input(scan(text,1,' '),best.)+60*input(scan(text,3,' '),best.);
 otherwise 
  time = 60*input(scan(text,1,' '),best.);
end;
output;

%if &type ^= tr %then %do;
keep id addr1 addr2 distance units time;
label
addr1 = 'ADDRESS #1'
addr2 = 'ADDRESS #2'
id    = 'ID #'
distance = 'DISTANCE'
time     = 'TIME (HR:MIN)'
;
%end;
%else %do;
keep id addr1 addr2 time;
label
addr1 = 'ADDRESS #1'
addr2 = 'ADDRESS #2'
id    = 'ID #'
time     = 'TIME (HR:MIN)'
;
%end;

stop;
done:
output;
run;
 
filename x clear;
filename z clear;
 
proc append base=distance_time data=temp;
run;
%end;
%mend;

* use the macro ... () DRIVING, (w) WALKING, (b) BICYCLING, (tr) TRANSIT TIME;
%distance_time();

 
* add the distance between ZIP centroids;
data distance_time;
set distance_time;
* ZIP should be LAST entry in the addresses;
zip_city = zipcitydistance(scan(addr1,-1),scan(addr2,-1));
label zip_city = 'MILES ZIPCITYDISTANCE FUNCTION';
run;

* look at results for DRIVING, WALKING, BICYCLING;
title "DISTANCES AND TIMES";
proc print data=distance_time label noobs;
var id addr1 addr2  distance units zip_city time;
format time time6.;
run;

* look at results for PUBLIC TRANSIT;
title "TIMES";
proc print data=distance_time label noobs;
var id addr1 addr2  time zip_city;
format time time6.;
run;
